package com.cours.helper;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.cours.annotations.MyAnnotation;
import com.cours.examples.Vehicule;
import com.cours.examples.Voiture;

public class ReflexiveHelper {

	private static final Log log = LogFactory.getLog(ReflexiveHelper.class);

	public ReflexiveHelper() {

	}

	public void displayStringClassInformations() {
		Class<?> myClassString = null;
		try {
			myClassString = new String().getClass();
			log.debug("1--SimpleName : " + myClassString.getSimpleName());
			log.debug("1--Name : " + myClassString.getName());
			myClassString = String.class;
			log.debug("2--SimpleName : " + myClassString.getSimpleName());
			log.debug("2--Name : " + myClassString.getName());
			myClassString = Class.forName("java.lang.String");
			log.debug("3--SimpleName : " + myClassString.getSimpleName());
			log.debug("3--Name : " + myClassString.getName());
		} catch (Exception e) {
			log.error("Une erreur s'est produite lors de l'execution la methode, Exception : " + e.getMessage());
		}
	}

	public void displayClassInformations(Object myObject) {
		try {
			Class<?> myObjectClass = myObject.getClass();
			log.debug("myObject SimpleName : " + myObjectClass.getSimpleName());
			log.debug("myObject Name : " + myObjectClass.getName());

			myObjectClass = Voiture.class;
			log.debug("Voiture SimpleName : " + myObjectClass.getSimpleName());
			log.debug("Voiture Name : " + myObjectClass.getName());

			myObjectClass = Class.forName("com.cours.examples.Voiture");
			log.debug("Voiture forName SimpleName : " + myObjectClass.getSimpleName());
			log.debug("Voiture forName Name : " + myObjectClass.getName());
		} catch (Exception e) {
			log.error("Une erreur s'est produite lors de l'execution la methode, Exception : " + e.getMessage());
		}
	}

	public void displaySuperClasses(Object myObject) {
		Class<?> myClass = myObject.getClass();
		log.debug("SimpleName : " + myClass.getSimpleName());
		log.debug("Superclass : " + myClass.getSuperclass());
		while ((myClass = myClass.getSuperclass()) != null) {
			log.debug("Name : " + myClass.getName());
		}
	}

	public void displayInterfaces(Object myObject) {
		Class<?> myClass = myObject.getClass();
		log.debug("SimpleName : " + myClass.getSimpleName());
		for (Class<?> oneClass : myClass.getInterfaces()) {
			log.debug("Name : " + oneClass.getName());
		}
	}

	public void displayPackage(Object myObject) {
		Class<?> myClass = myObject.getClass();
		log.debug("Package : " + myClass.getPackage());
	}

	public void displayMethodInformations(Object myObject, String methodName, Class<?>... parametersTypes) {
		Class<?> myClass = myObject.getClass();
		log.debug("SimpleName : " + myClass.getSimpleName());
		for (Method oneMethod : myClass.getMethods()) {
			log.debug("MethodName : " + oneMethod.getName());
		}
		try {
			Method myMethod = myClass.getMethod(methodName, parametersTypes);
			log.debug("myMethod : " + myMethod.getName());
		} catch (Exception e) {
			log.error("Une erreur s'est produite lors de l'execution la methode, Exception : " + e.getMessage()
					+ " , Exception : " + e);
		}
	}

	public void displayAllFields(Object myObject) {
		Class<?> myClass = myObject.getClass();
		log.debug("SimpleName : " + myClass.getSimpleName());
		for (Field field : myClass.getFields()) {
			log.debug("Name : " + field.getName());
		}
	}

	public void displayAllDeclaredFields(Object myObject) {
		Class<?> myClass = myObject.getClass();
		log.debug("SimpleName : " + myClass.getSimpleName());
		for (Field field : myClass.getDeclaredFields()) {
			log.debug("Name : " + field.getName());
		}
	}

	public void displayAllConstructors(Object myObject, Class<?>[] parameterType) {
		Class<?> myClass = myObject.getClass();
		log.debug("SimpleName : " + myClass.getSimpleName());
		for (Constructor<?> myConstructor : myClass.getConstructors()) {
			log.debug("Name : " + myConstructor.getName());
		}
		try {
			Constructor<?> myConstructor = myClass.getConstructor(parameterType);
			log.debug("myConstructor : " + myConstructor.getName());
		} catch (Exception e) {
			log.error("Une erreur s'est produite lors de l'execution la methode, Exception : " + e.getMessage()
					+ " , Exception : " + e);
		}
	}

	public void instanciateVehiculeWithoutArgument() {
		// Standard
		Vehicule vehicule = new Vehicule();
		log.debug("vehicule : " + vehicule);
		try {
			// Reflexive
			Vehicule myReflexiveVehicule = null;
			Class<?> myClass = Class.forName("com.cours.examples.Vehicule");
			Object myObjectVehicule = myClass.newInstance();
			log.debug("TypeName : " + myObjectVehicule.getClass().getTypeName());
			if (myObjectVehicule instanceof Vehicule) {
				myReflexiveVehicule = (Vehicule) myObjectVehicule;
			}
			log.debug("myReflexiveVehicule : " + myReflexiveVehicule);
		} catch (Exception e) {
			log.error("Une erreur s'est produite lors de l'execution la methode, Exception : " + e.getMessage()
					+ " , Exception : " + e);
		}
	}

	public void instanciateVehiculeWithArgument(String marque, String couleur, String className) {
		// Standard
		Vehicule bmw = new Vehicule(marque, couleur);
		log.debug("bmw : " + bmw);
		try {
			// Reflexive
			Vehicule myReflexiveBMW = null;
			Class<?> myClass = Class.forName(className);
			Class<?>[] parameterType = new Class[] { String.class, String.class };
			Constructor<?> myConstructor = myClass.getConstructor(parameterType);
			Object[] parametersValue = new Object[] { marque, couleur };
			Object myObjectBMW = myConstructor.newInstance(parametersValue);
			if (myObjectBMW instanceof Vehicule) {
				myReflexiveBMW = (Vehicule) myObjectBMW;
			}
			log.debug("myReflexiveBMW : " + myReflexiveBMW);
		} catch (Exception e) {
			log.error("Une erreur s'est produite lors de l'execution la methode, Exception : " + e.getMessage()
					+ " , Exception : " + e);
		}
	}

	public void instanciateVehiculeWithArgumentBis() {
		// Standard
		Vehicule bmw = new Vehicule("BMW", "Noire");
		log.debug("bmw : " + bmw);
		try {
			// Reflexive
			Vehicule myReflexiveBMW = null;
			Class<?> myClass = Class.forName("com.cours.examples.Vehicule");
			Class<?>[] parameterType = new Class[] { String.class, String.class };
			Constructor<?> myConstructor = myClass.getConstructor(parameterType);
			Object[] parametersValue = new Object[] { "BMW", "Noire" };
			Object myObjectBMW = myConstructor.newInstance(parametersValue);
			if (myObjectBMW instanceof Vehicule) {
				myReflexiveBMW = (Vehicule) myObjectBMW;
			}
			log.debug("myReflexiveBMW : " + myReflexiveBMW);
		} catch (Exception e) {
			log.error("Une erreur s'est produite lors de l'execution la methode, Exception : " + e.getMessage()
					+ " , Exception : " + e);
		}
	}

	public void modifyVehicule(Object myObject, String marque, String couleur) {
		Class<?> myClass = null;
		try {
			myClass = myObject.getClass();
			Class<?>[] parameterType = new Class[] { String.class };
			Method mySetMarqueMethod = myClass.getMethod("setMarque", parameterType);
			log.debug("myMethod : " + mySetMarqueMethod.getName());
			Object[] myParamSetMarque = new Object[] { marque };
			mySetMarqueMethod.invoke(myObject, myParamSetMarque);

			Method mySetCouleurMethod = myClass.getMethod("setCouleur", parameterType);
			log.debug("myMethod : " + mySetCouleurMethod.getName());
			Object[] myParamSetCouleur = new Object[] { couleur };
			mySetCouleurMethod.invoke(myObject, myParamSetCouleur);

			Vehicule myReflexiveVehicule = null;
			if (myObject instanceof Vehicule) {
				myReflexiveVehicule = (Vehicule) myObject;
				myReflexiveVehicule.descriptionVehicule();
			}
			log.debug("myReflexiveVehicule : " + myReflexiveVehicule);
		} catch (Exception e) {
			log.error("Une erreur s'est produite lors de l'execution la methode, Exception : " + e.getMessage()
					+ " , Exception : " + e);
		}
	}

	public void invokeStaticMethod(String className, String marque, String couleur) {
		Class<?> myClass = null;
		try {
			myClass = Class.forName(className);
			Class<?>[] parameterType = new Class[] { String.class, String.class };
			Method myMethod = myClass.getMethod("getMarqueCouleur", parameterType);
			log.debug("myMethod : " + myMethod.getName());
			Object[] myParam = new Object[] { marque, couleur };
			myMethod.invoke(null, myParam);
		} catch (Exception e) {
			log.error("Une erreur s'est produite lors de l'execution la methode, Exception : " + e.getMessage()
					+ " , Exception : " + e);
		}
	}

	public void invokeStaticMethodAbsMath() {
		Integer myNumber = null;
		try {
			// Standard
			myNumber = Math.abs(-1);
			log.debug("Standard myNumber : " + myNumber);

			// De façon Reflexive
			Class<?>[] typeAbs = new Class[] { int.class };
			Object[] paramAbs = new Object[] { -1 };

			Method myMethodAbs = Math.class.getMethod("abs", typeAbs);
			Object myNumberObject = myMethodAbs.invoke(null, paramAbs);
			if (myNumberObject instanceof Integer) {
				myNumber = (Integer) myNumberObject;
				log.debug("Reflexive myNumber : " + myNumber);
			}
		} catch (Exception e) {
			log.error("Une erreur s'est produite lors de l'execution la methode, Exception : " + e.getMessage()
					+ " , Exception : " + e);
		}
	}

	public List<Field> getFieldsWithAnnotation(Object myObject) {
		List<Field> fields = new ArrayList<Field>();
		Class<?> myClass = myObject.getClass();
		Object myValue = null;
		log.debug("SimpleName : " + myClass.getSimpleName());
		for (Field currentField : myClass.getDeclaredFields()) {
			if (currentField.isAnnotationPresent(MyAnnotation.class)) {
				log.debug("Name : " + currentField.getName());
				fields.add(currentField);
			}
		}
		try {
			boolean isAccessible = false;
			for (Field currentField : fields) {
				isAccessible = currentField.isAccessible();
				if (!isAccessible) {
					currentField.setAccessible(true);
				}
				myValue = currentField.get(myObject);
				currentField.setAccessible(isAccessible);
				log.debug("MyAnnotation Name : " + currentField.getName() + " , myValue : " + myValue);
			}
		} catch (Exception e) {
			log.error("Une erreur s'est produite lors de l'execution la methode, Exception : " + e.getMessage()
					+ " , Exception : " + e);
		}
		return fields;
	}
}
